# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

from autoENRICH.util.labelmaker import labelmaker
from autoENRICH.reference.periodic_table import Get_periodic_table

# Write an nmrmol object to an nmredata file
def nmrmol_to_nmredata(mol, outfile):

	# Get periodic table
	periodic_table = Get_periodic_table()

	# Assume molecule is not chiral
	chiral = 0
	# Determine number of bonds
	bonds = 0
	for at1 in range(len(mol.types)):
		for at2 in range(at1, len(mol.types)):

			if mol.conn[at1][at2] >= 1:
				bonds += 1

	atoms = len(mol.types)

	# Start putting together file lines
	lines = []
	# print molecule name
	if '/' in outfile:
		lines.append(outfile.split('/')[-1].split('.')[0])
	else:
		lines.append(outfile.split('.')[0])
	# print file and author
	lines.append('auto-ENRICH - 2019 - Will Gerrard')
	lines.append('')

	# Structure section
	string = '{atoms:>3d}{bonds:>3d}  0  0{chiral:>3d}  0  0  0  0  0  1 V2000'.format(atoms=atoms,
																							bonds=bonds,
																							chiral=chiral)
	lines.append(string)
	# Print xyz coordinates and types
	for i, xyz in enumerate(mol.xyz):
		string = '{x:>10.4f}{y:>10.4f}{z:>10.4f} {typechar:>3s} 0  0  0  0  0  0  0  0  0  0  0  0'.format(x=xyz[0],
																												y=xyz[1],
																												z=xyz[2],
																												typechar = periodic_table[mol.types[i]])
		lines.append(string)
	# Print bonds and bond types
	for at1 in range(len(mol.types)):
		for at2 in range(at1, len(mol.types)):

			if mol.conn[at1][at2] >= 1:
				string = '{at1:>3d}{at2:>3d}{bond:>3d}  0  0  0  0'.format(at1=at1+1,
																				at2=at2+1,
																				bond=mol.conn[at1][at2])
				lines.append(string)
	# Terminate structure section
	lines.append('M\tEND'.format())


	# assignment section
	lines.append('')
	lines.append('> <NMREDATA_ASSIGNMENT>')
	# Print chemical shifts with variance
	for i, shift, type, var in zip(range(len(mol.types)), mol.shift, mol.types, mol.shift_var):
		string = " {atom:<5d}, {shift:<15.8f}, {type:<5d}, {variance:<15.8f}\\".format(atom=i, shift=shift, type=type, variance=var)
		lines.append(string)

	lines.append('')
	lines.append('> <NMREDATA_J>')
	# Print couplings with variance and label
	for i in range(len(mol.types)):
		for j in range(len(mol.types)):
			if i >= j:
				continue
			if mol.coupling_len[i][j] == 0:
				continue
			label = labelmaker(i, j, mol)
			string = " {a1:<10d}, {a2:<10d}, {coupling:<15.8f}, {label:<10s}, {var:<15.8f}".format(a1=i,
																									a2=j,
																									coupling=mol.coupling[i][j],
																									label=label,
																									var=mol.coupling_var[i][j])

			lines.append(string)

	# Print assembled lines to output file
	with open(outfile, 'w') as f:
		for line in lines:
			print(line, file=f)



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

from autoENRICH.file_creation import HPC_submission as HPCsub
import autoENRICH.file_read.orca_read as orcaread
import autoENRICH.file_creation.orca_submission as orcasub
import autoENRICH.file_creation.structure_formats.nmredata as nmredata
import sys

def setup_nmr(molecule, prefs, path='', ids=[], max=50):

	nmr_files = []
	for conformer in molecule.conformers:
		if (conformer.nmr_status == 'None' or conformer.nmr_status == 'pre-submission') and conformer.opt_status == 'successful':
			conformer.nmr_in = orcasub.make_nmrin(prefs, conformer.molid, conformer.xyz, conformer.types,
														path + 'NMR/')
			conformer.nmr_log = conformer.nmr_in.split('.')[0] + '.log'
			conformer.nmr_status = 'pre-submission'
			nmr_files.append('NMR/' + conformer.nmr_in.split('/')[-1])

	if len(nmr_files) == 0:
		print('No files to submit. . .')
		sys.exit(0)

	IN_ARRAY = 'NMR/NMR_IN_ARRAY.txt'
	with open(path + IN_ARRAY, 'w') as f:
		for file in nmr_files:
			print(file, file=f)

	system = prefs['comp']['system']
	memory = prefs['NMR']['memory']
	processors = prefs['NMR']['processors']
	walltime = prefs['NMR']['walltime']

	files = len(nmr_files)
	chunks = HPCsub.get_chunks(files)
	qsub_names = []
	for ck in range(chunks):
		start = (ck * max) + 1
		end = ((ck + 1) * max)
		if end > files:
			end = files


		#header = HPCsub.make_HPC_header(jobname=jobname, system=system, nodes=1, ppn=processors, walltime=walltime, mem=memory)
		jobname = 'aE_' + molecule.molid + '_' + str(ck) + '_NMR'
		strings = HPCsub.make_HPC_orca_batch_submission(prefs, molecule.molid, IN_ARRAY, start, end, ck=ck,
									jobname=jobname, nodes=1, ppn=processors, walltime=walltime, mem=memory)

		if prefs['comp']['system'] == 'BC3':
			filename = path + 'NMR_' + molecule.molid + '_' + str(ck) + '.qsub'
		elif prefs['comp']['system'] == 'BC4':
			filename = path + 'NMR_' + molecule.molid + '_' + str(ck) + '.slurm'
		elif prefs['comp']['system'] == 'localbox':
			filename = path + 'NMR_' + molecule.molid + '_' + str(ck) + '.sh'
		with open(filename, 'w') as f:
			for string in strings:
				print(string, file=f)
		qsub_names.append(filename)

	print('Created ', chunks, ' submission files. . .')
	if prefs['comp']['system'] == 'BC3':
		print('Submit the calculations using:')
		for file in qsub_names:
			print('qsub ', file)
	elif prefs['comp']['system'] == 'Grendel':
		print('Submit the calculations using:')
		for file in qsub_names:
			print('bash ', file)
	elif prefs['comp']['system'] == 'BC4':
		print('Havent finished this yet, good luck pal. . . .')


def process_nmr(molecule, prefs, path=''):

	good = 0
	bad = 0
	process = False

	for conformer in molecule.conformers:
		status = orcaread.get_nmr_status(conformer.nmr_log)
		if status == 'successful':
			good +=1
			conformer.read_nmr(conformer.nmr_log, type='orca')
		else:
			bad += 1

		if conformer.nmr_status != status:
			process = True
			outname = path + 'OUTPUT/' + molecule.molid + '_' + conformer.molid + '.nmredata.sdf'
			nmredata.nmrmol_to_nmredata(molecule, outname)

		conformer.nmr_status = status
		string = 'Conformer {molid:^3s} status: {status:^10s}'.format(molid=str(conformer.molid),
																								status=conformer.nmr_status)
		print(string)

	if process:
		molecule.boltzmann_average()
		outname = path + 'OUTPUT/' + molecule.molid + '.nmredata.sdf'
		nmredata.nmrmol_to_nmredata(molecule, outname)

	print(good, ' successful NMR calculations, ', bad, ' failed, out of ', len(statuss))















###



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

from . import HPS_generic as generic
import numpy as np
import itertools
import pickle
import sys

def grid_search(dataset, args):

	# define grid
	search_grid = {}
	search_list = []
	size_list = []

	# determine whether log dictionary was provided
	if len(args['param_logs']) == 0:
		check_logs = False
	else:
		check_logs = True

	# create search grid
	for param in args['param_ranges']:
		if args['param_logs'][param] == 'no':
			continue
		if check_logs:
			if args['param_logs'][param] == 'log':
				search_grid[param] = np.logspace(args['param_ranges'][param][0], args['param_ranges'][param][1], args['grid_density'], endpoint=True)
			else:
				search_grid[param] = np.linspace(args['param_ranges'][param][0], args['param_ranges'][param][1], args['grid_density'], endpoint=True)
		else:
			search_grid[param] = np.linspace(args['param_ranges'][param][0], args['param_ranges'][param][1], args['grid_density'], endpoint=True)

	# create list of parameter sets to test
	for i in range(args['grid_density'] ** len(args['param_ranges'].keys())):
		param_point = {}
		xx = 0
		for param in search_grid.keys():
			idx = int(i / args['grid_density'] ** xx) % args['grid_density']
			param_point[param] = search_grid[param][idx]
			xx += 1
		search_list.append(param_point)

	generic.setup_logfile(args)

	BEST_SCORE = 999.9999999
	BEST_PARAMS = {}
	for p, params in enumerate(search_list):

		score, BEST_SCORE, BEST_PARAMS = generic.HPS_iteration(p, dataset, args, next_point_to_probe=params,
															BEST_SCORE=BEST_SCORE, BEST_PARAMS=BEST_PARAMS)

	outname = generic.save_models(dataset, BEST_PARAMS, args)
	print('Optimised model saved in ', outname)

	return dataset, BEST_SCORE















###



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

# Get status of ORCA optimisation
def get_opt_status(file):
	# Input:
	#	file: filename

	# Returns: status

	status = 'unknown'
	try:
		with open(file, 'r') as f:
			for line in f:
				if 'SUCCESS' in line:
					status = 'successful'
				if 'ERROR' in line:
					status = 'failed'
	except Exception as e:
		status = 'not submitted'

	return status

# Get status of ORCA NMR calculation
def get_nmr_status(file):
		# Input:
		#	file: filename

		# Returns: status
	status = 'unknown'
	with open(file, 'r') as f:
		for line in f:
			if 'SUCCESS' in line:
				status = 'successful'
			if 'ERROR' in line:
				status = 'failed'
	return status

# Read Optimisation energy from ORCA optimisation file
def read_opt(file):
	# Input:
	#	file: filename
	# Returns: energy

	# Go through file looking for this line
	# SCF Energy:    -1072.8219232141
	energy = 0.0
	with open(file ,'r') as f:
		for line in f:
			if 'SCF Energy:' in line:
				items=line.split()
				energy = float(items[-1])

	return energy

# Need to write this
def read_nmr(file):



	return shift, coupling



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

def construct_param_dict(modelflag, featureflag, targetflag):

	params = {}
	param_ranges = {}
	param_logs = {}

	if modelflag == 'KRR':
		param_ranges['sigma'] = [-5, 5]
		param_logs['sigma'] = 'log'

		param_ranges['lamda'] = [-10, 1]
		param_logs['lamda'] = 'log'

	elif modelflag == 'FCHL':
		param_ranges['sigma'] = [-5, 5]
		param_logs['sigma'] = 'log'

		param_ranges['lamda'] = [-10, 1]
		param_logs['lamda'] = 'log'

		param_ranges['two_body_scaling'] = [1, 4]
		param_logs['two_body_scaling'] = 'lin'

		param_ranges['three_body_scaling'] = [1, 4]
		param_logs['three_body_scaling'] = 'lin'

		param_ranges['two_body_width'] = [0, 3]
		param_logs['two_body_width'] = 'lin'

		param_ranges['three_body_width'] = [1, 4]
		param_logs['three_body_width'] = 'lin'

		param_ranges['three_body_power'] = [1, 4]
		param_logs['three_body_power'] = 'lin'

		param_ranges['two_body_power'] = [2, 5]
		param_logs['two_body_power'] = 'lin'

		param_ranges['cut_start'] = [0, 3]
		param_logs['cut_start'] = 'lin'

		param_ranges['alchemy_period_width'] = [1, 4]
		param_logs['alchemy_period_width'] = 'lin'

		param_ranges['alchemy_group_width'] = [1, 4]
		param_logs['alchemy_group_width'] = 'lin'

	elif modelflag == 'NN':
		param_ranges['hidden_neurons'] = [1, 100]
		param_logs['hidden_neurons'] = 'lin'

		param_ranges['hidden_layers'] = [1, 10]
		param_logs['hidden_layers'] = 'lin'

		param_ranges['learning_rate'] = [-5, 2]
		param_logs['learning_rate'] = 'log'

		param_ranges['nn_epochs'] = [1, 100]
		param_logs['nn_epochs'] = 'lin'

		param_ranges['batch_size'] = [1, 100]
		param_logs ['batch_size'] = 'lin'

		pass

	elif modelflag == 'TFM':
		param_ranges['d_model'] = [400, 800]
		param_logs['d_model'] = 'lin'

		param_ranges['n_layer'] = [10, 18]
		param_logs['n_layer'] = 'lin'

		param_ranges['d_inner'] = [3700, 3900]
		param_logs['d_inner'] = 'lin'

		param_ranges['feature_dim'] = [199.9, 200.1]
		param_logs['feature_dim'] = 'lin'

		param_ranges['final_dim'] = [279.9, 280.1]
		param_logs['final_dim'] = 'lin'

		param_ranges['dropout'] = [0.02, 0.04]
		param_logs['dropout'] = 'lin'

		param_ranges['dropatt'] = [0.0, 0.01]
		param_logs['dropatt'] = 'lin'

		param_ranges['final_dropout'] = [0.03, 0.04]
		param_logs['final_dropout'] = 'lin'

		param_ranges['n_head'] = [9, 11]
		param_logs['n_head'] = 'lin'

		param_ranges['eta_min'] = [-6, -7]
		param_logs['eta_min'] = 'log'

		param_ranges['learning_rate'] = [0.0009, 0.0011]
		param_logs['learning_rate'] = 'lin'


	if featureflag == 'CMAT':
		param_ranges['cutoff'] = [1, 10]
		param_logs['cutoff'] = 'lin'

		param_ranges['central_decay'] = [0, 10]
		param_logs['central_decay'] = 'lin'

		param_ranges['interaction_cutoff'] = [0, 10]
		param_logs['interaction_cutoff'] = 'lin'

		param_ranges['interaction_decay'] = [0, 10]
		param_logs['interaction_decay'] = 'lin'


	elif featureflag == 'aSLATM':
		param_ranges['cutoff'] = [1, 10]
		param_logs['cutoff'] = 'lin'

	elif featureflag == 'FCHL':
		param_ranges['cutoff'] = [1, 10]
		param_logs['cutoff'] = 'lin'

	elif featureflag == 'ACSF':
		param_ranges['cutoff'] = [1, 10]
		param_logs['cutoff'] = 'lin'

		param_ranges['nRs2'] = [1, 10]
		param_logs['nRs2'] = 'lin'

		param_ranges['nRs3'] = [1, 10]
		param_logs['nRs3'] = 'lin'

		param_ranges['nTs'] = [1, 10]
		param_logs['nTs'] = 'lin'

		param_ranges['eta2'] = [0, 10]
		param_logs['eta2'] = 'lin'

		param_ranges['eta3'] = [0, 10]
		param_logs['eta3'] = 'lin'

		param_ranges['zeta'] = [0, 10]
		param_logs['zeta'] = 'lin'

		param_ranges['acut'] = [1, 10]
		param_logs['acut'] = 'lin'

		param_ranges['bin_min'] = [0, 10]
		param_logs['bin_min'] = 'lin'

	elif featureflag == 'BCAI':
		param_ranges['cutoff'] = [0, 10]
		param_logs['cutoff'] = 'lin'


	return param_ranges, param_logs






























###



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

from autoENRICH.reference.periodic_table import Get_periodic_table

def labelmaker(i, j, mol):
	Periodic_table = Get_periodic_table()
	lent = mol.coupling_len[i][j]
	label = str(lent) + str('J')
	if mol.types[int(i)] >= mol.types[int(j)]:
		label = label + str(Periodic_table[mol.types[int(i)]]) + str(Periodic_table[mol.types[int(j)]])
	else:
		label = label + str(Periodic_table[mol.types[int(j)]]) + str(Periodic_table[mol.types[int(i)]])

	return label



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.


from autoENRICH.file_creation.HPC_submission import make_HPC_header

import pickle
import glob
import sys

# creates a submission file that will run a hyper parameter search
def setup_trainmodel(args):

	# NEED TO MAKE GPU VERSION
	python_env = args['python_env']
	system = args['system']
	nodes = 1
	ppn = args['processors']
	walltime = args['walltime']
	mem = args['memory']

	jobname = 'IMP_' + args['modelflag'] + '_' + args['featureflag'] + '_' + args['targetflag'] + '_' + args['searchflag'] + '_HPS'
	submission_file = jobname + '.submit'
	header = make_HPC_header(jobname, system=args['system'], nodes=1, ppn=args['processors'], walltime=args['walltime'], mem=args['memory'])
	strings = []
	strings.append('')
	strings.append('shopt -s expand_aliases')
	strings.append('source ~/.bashrc')
	strings.append("source activate {env:<10s}".format(env=python_env))
	strings.append("IMPRESSION train --prefs {0:<10s}".format(args['prefs']))


	with open(submission_file, 'w') as f:
		for string in header:
			print(string, file=f)
		for string in strings:
			print(string, file=f)


def trainmodel(args):

	from autoENRICH.ml.HPS import HPS
	from autoENRICH.molecule.dataset import dataset
	from autoENRICH.molecule.nmrmol import nmrmol

	params = []
	for param in args['param_ranges'].keys():
		params.append(param)
	args['param_list'] = params

	if args['feature_optimisation'] == 'True':
		if args['training_set'].split('.')[-1] == 'pkl':
			dset = pickle.load(open(args['training_set'], 'rb'))
		elif args['training_set'].split('.')[-1] == 'csv':
			dset = load_dataset_from_csv(args['training_set'])
		else:
			files = glob.glob(args['training_set'])
			dset = dataset()
			dset.get_mols(files, type='nmredata')
			assert len(dset.mols) > 0

			if args['store_datasets'] == 'True':
				pickle.dump(dset, open('training_set.pkl', 'wb'))

		dset, score = HPS(dset, args)

		if args['store_datasets'] == 'True':
			pickle.dump(dset, open('OPT_training_set.pkl', 'wb'))


	else:
		dset = pickle.load(open(args['feature_file'], "rb"))
		assert len(dset.x) > 0
		assert len(dset.y) > 0

		HPS(dset, args)





###



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

import json
import re
import sys
import os

def Get_tantillo_factors(basis_set='6-311g(d,p)', functional='wb97xd'):
	'''
	tantillo=	{}
	tantillo['functional'] = functional
	tantillo['basis_set'] = basis_set
	tantillo['H'] = [-1.0719, 32.1254]
	tantillo['C'] = [-1.0399, 187.136]

	dir_path = os.path.dirname(os.path.realpath(__file__))
	json_file = dir_path + '/scaling_factors/' + re.sub(r'\W+', '', functional+basis_set) + '.json'
	json.dump(tantillo, open(json_file, 'w'), indent=4)
	'''
	try:
		dir_path = os.path.dirname(os.path.realpath(__file__))
		json_file = dir_path + '/scaling_factors/' + re.sub(r'\W+', '', functional+'_'+basis_set) + '.json'
		tantillo = json.load(open(json_file, 'r'))
	except Exception as e:
		print(e)
		print('Couldnt load scaling factor json file. . . ')
		sys.exit(0)

	return tantillo



def Calculate_new_factors(exp_mols, dft_mols, functional, basis_set):
	print('STUB, havent written this yet')
	# do basic linear regression


	dir_path = os.path.dirname(os.path.realpath(__file__))
	json_file = dir_path + '/scaling_factors/' + re.sub(r'\W+', '', functional+'_'+basis_set) + '.json'
	json.dump(tantillo, open(json_file, 'w'), indent=4)






















#



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
     name='autoenrich',
     version='2.1.1',
     scripts=['bin/autoenrich', 'bin/impression', 'bin/ae_utils'] ,
     author="Will Gerrard",
     author_email="will.gerrard@bristol.ac.uk",
     description="Computational NMR Library",
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/wg12385/autoenrich_public",
     packages=setuptools.find_namespace_packages(exclude=['_site/*', 'bin/*', 'build/*', 'dist/*', 'tests/*', '*test*', 'dockerfiles/*', 'setup.py']),
	 classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
         "Operating System :: OS Independent",
     ]
 )



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.


def Get_periodic_table():
    periodic_table=[
        '',
        'H',
        'He',
        'Li',
        'Be',
        'B',
        'C',
        'N',
        'O',
        'F',
        'Ne',
        'Na',
        'Mg',
        'Al',
        'Si',
        'P',
        'S',
        'Cl',
        'Ar',
        'K',
        'Ca',
        'Sc',
        'Ti',
        'V',
        'Cr',
        'Mn',
        'Fe',
        'Co',
        'Ni',
        'Cu',
        'Zn',
        'Ga',
        'Ge',
        'As',
        'Se',
        'Br',
        'Kr',
        'Rb',
        'Sr',
        'Y',
        'Zr',
        'Nb',
        'Mo',
        'Tc',
        'Ru',
        'Rh',
        'Pd',
        'Ag',
        'Cd',
        'In',
        'Sn',
        'Sb',
        'Te',
        'I',
        'Xe',
        'Cs',
        'Ba',
        'La',
        'Ce',
        'Pr',
        'Nd',
        'Pm',
        'Sm',
        'Eu',
        'Gd',
        'Tb',
        'Dy',
        'Ho',
        'Er',
        'Tm',
        'Yb',
        'Lu',
        'Hf',
        'Ta',
        'W',
        'Re',
        'Os',
        'Ir',
        'Pt',
        'Au',
        'Hg',
        'Tl',
        'Pb',
        'Bi',
        'Po',
        'At',
        'Rn',
        'Fr',
        'Ra',
        'Ac',
        'Th',
        'Pa',
        'U',
        'Np',
        'Pu',
        'Am',
        'Cm',
        'Bk',
        'Cf',
        'Es',
        'Fm',
        'Md',
        'No',
        'Lr',
        'Rf',
        'Db',
        'Sg',
        'Bh',
        'Hs',
        'Mt',
        'Ds',
        'Rg',
        'Cn',
        'Nh',
        'Fl',
        'Mc',
        'Lv',
        'Ts',
        'Og',
        ]
    return periodic_table



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

# make orca submission files
from autoENRICH.reference.periodic_table import Get_periodic_table

#
def make_optin(prefs, molname, xyz, types, path=''):
	# Input:
	#	prefs: preferences dictionary
	#	molname: name of molecule
	#	xyz: xyz coordinates of conformer
	#	types: type list of conformer (numeric)
	#	path: path to molecule folder

	# Returns: filename for input file

	# Get preferences from prefs
	charge = prefs['mol']['charge']
	multiplicity = prefs['mol']['multiplicity']
	functional = prefs['optimisation']['functional']
	basis_set = prefs['optimisation']['basisset']
	solvent = prefs['optimisation']['solvent']
	direct_cmd_line_opt = prefs['optimisation']['custom_cmd_line']
	# Get periodic table
	Periodic_table = Get_periodic_table()
	# Define instruction line for ORCA
	instr = '! ' + str(functional) + ' ' + str(basis_set) + ' TightSCF OPT miniprint'
	# Add parallel option if multiple processors requested
	if processors != 1:
		instr += ' PAL{0:<d}'.format(processors)
	# Add solvent model/solvent if requested
	if solvent != 'none':
		instr += ' CPCM(' + solvent + ')'
	# If direct line input specified then overwrite all of this
	if direct_cmd_line_opt:
		instr = direct_cmd_line_opt

	# Define input file path/name
	infile = path.strip() + molname.strip() + '_OPT.in'
	# Construct file strings
	strings = []
	strings.append(instr)
	strings.append('')
	strings.append("* xyz {0:<1d} {1:<1d}".format(charge, multiplicity))
	for i in range(len(xyz)):
		str_type = Periodic_table[types[i]]
		string = " {0:<2s}        {1:>10.5f}        {2:>10.5f}        {3:>10.5f}".format(str_type, xyz[i][0], xyz[i][1], xyz[i][2])
		strings.append(string)
	strings.append('*')
	strings.append('')
	strings.append('%geom')
	strings.append('     AddExtraBonds true         # switch on/off assigning bonds to atom pairs that are')
	strings.append('                                #  connected by more than <Max_Length> bonds and are less')
	strings.append('                                #  than <MaxDist> Ang. apart (default true)')
	strings.append('     AddExtraBonds_MaxLength 10 # cutoff for number of bonds connecting the two')
	strings.append('                                #  atoms (default 10)')
	strings.append('     AddExtraBonds_MaxDist 5    # cutoff for distance between two atoms (default 5 Ang.)')
	strings.append('end')
	# Write file
	with open(infile, 'w') as f_handle:
		for string in strings:
			print(string, file=f_handle)

	return infile

def make_nmrin(prefs, molname, xyz, types, path=''):
	# Input:
	#	prefs: preferences dictionary
	#	molname: name of molecule
	#	xyz: xyz coordinates of conformer
	#	types: type list of conformer (numeric)
	#	path: path to molecule folder

	# Returns: input file path/name

	# Get values from preferences
	charge = prefs['mol']['charge']
	multiplicity = prefs['mol']['multiplicity']
	functional = prefs['NMR']['functional']
	basis_set = prefs['NMR']['basisset']
	aux_basis_set = prefs['NMR']['aux_basis_set']
	solvent = prefs['NMR']['solvent']
	direct_cmd_line_nmr = prefs['NMR']['custom_cmd_line']
	processors = prefs['NMR']['processors']
	# Get periodic table
	Periodic_table = Get_periodic_table()
	# Construct instruction line for ORCE
	instr = '! ' + str(functional) + ' ' + str(basis_set) + ' ' + str(aux_basis_set) +  '  TightSCF miniprint' + ' NMR '
	# Add parallel option if multiple processors requested
	if processors != 1:
		instr += ' PAL{0:<d}'.format(processors)
	# Add solvent model/solvent if requested
	if solvent != 'none':
		instr += ' CPCM(' + solvent + ')'
	# If direct line input specified then overwrite all of this
	if direct_cmd_line_nmr:
		instr = direct_cmd_line_nmr
	# Define input file path/name
	infile = path.strip() + molname.strip() + '_NMR.in'
	# Construct file strings
	strings = []
	strings.append(instr)
	strings.append("")
	strings.append("* xyz {0:<1d} {1:<1d}".format(charge, multiplicity))
	for i in range(len(xyz)):
		str_type = Periodic_table[types[i]]
		string = " {0:<2s}        {1:>10.6f}        {2:>10.6f}        {3:>10.6f}".format(str_type, xyz[i][0], xyz[i][1], xyz[i][2])
		strings.append(string)
	strings.append('*')
	strings.append('%eprnmr')
	for type in prefs['NMR']['shift_nuclei']:
		strings.append("       Nuclei = all {type:<2s}".format(type=type) + '  { shift }')
	for type in prefs['NMR']['spin_nuclei']:
		strings.append("       Nuclei = all {type:<2s}".format(type=type) + '  { ssall }')
	strings.append('SpinSpinRThresh {0:<f}'.format(prefs['NMR']['spin_thresh']))
	strings.append('end')
	# Write file
	with open(infile, 'w') as f_handle:
		for string in strings:
			print(string, file=f_handle)

	return infile



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

from autoENRICH.reference.periodic_table import Get_periodic_table

# Write an nmrmol object to an xyz file
def nmrmol_to_xyz(mol, outname, num=-404):
	periodic_table = Get_periodic_table()
	with open(outname, 'w') as f:
		print(len(mol.types), file=f)
		if num == -404:
			print(mol.molid, file=f)
		else:
			string = "{0:<10d}\t{1:<20s}".format(num, mol.molid)
			print(string, file=f)

		for i in range(len(mol.types)):
			string = "{i:<10s}\t{x:<10.6f}\t{y:<10.6f}\t{z:<10.6f}".format(i=periodic_table[mol.types[i]],
																			x=mol.xyz[i][0],
																			y=mol.xyz[i][1],
																			z=mol.xyz[i][2])
			print(string, file=f)



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

def get_type(filename):

	extension = filename.split('.')[-1]
	if extension == 'sdf':
		if len(filename.split('.')) > 2:
			if filename.split('.')[-2] == 'nmredata':
				type = 'nmredata'
			else:
				type = 'sdf'
		else:
			type = 'sdf'

	elif extension == 'xyz':
		type = 'xyz'

	elif extension == 'log':
		type = 'g09'

	elif extension == 'mol2':
		type = 'mol2'

	else:
		print('type not recognised for file ', filename, ' please check file and specify type')
		type = ''

	return type



# Copyright 2020 Will Gerrard
#This file is part of autoENRICH.

#autoENRICH is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#autoENRICH is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with autoENRICH.  If not, see <https://www.gnu.org/licenses/>.

import numpy as np
import openbabel
from autoENRICH.reference.periodic_table import Get_periodic_table

def mol_iswhole(mol):
	iswhole = True
	atoms = mol_getatoms(mol)
	for i in range(atoms):
		for j in range(atoms):
			if i == j:
				continue
			good = False

			while good == False:
				for length in range(0, atoms):
					paths = mol_find_all_paths(mol, i, j, length)
					if len(paths) > 0:
						good = True
						break
				if not good:
					iswhole = False
					return iswhole
	return iswhole

def mol_splitmol(mol):
	atoms = mol_getatoms(mol)
	structures = []
	assigned = []
	structures.append([])

	depth = 0
	while len(assigned) < atoms:

		for i in range(atoms):
			if i in assigned:
				continue

			if len(structures[depth]) == 0:
				structures[depth].append(i)
				assigned.append(i)

			for j in range(atoms):
				if j in assigned or j == i:
					continue

				for length in range(0, atoms):
					paths = mol_find_all_paths(mol, i, j, length)
					if len(paths) > 0:
						structures[depth].append(j)
						assigned.append(j)
						break
						continue

			structures.append([])
			depth += 1

	lent = 0
	for structure in structures:
		lent += len(structure)

	assert lent == atoms

	return structures
